@@ -32,11 +32,17 @@ from jsonpath_rw_ext import _string
 class ExtendedJsonPathLexer(lexer.JsonPathLexer):
     """Custom LALR-lexer for JsonPath"""
     literals = lexer.JsonPathLexer.literals + ['?', '@', '+', '*', '/', '-']
-    tokens = (parser.JsonPathLexer.tokens +
-              ['FILTER_OP', 'SORT_DIRECTION'])
+    tokens = (['BOOL'] +
+              parser.JsonPathLexer.tokens +
+              ['FILTER_OP', 'SORT_DIRECTION', 'FLOAT'])
 
     t_FILTER_OP = r'==?|<=|>=|!=|<|>'
 
+    def t_BOOL(self, t):
+        r'true|false'
+        t.value = True if t.value == 'true' else False
+        return t
+
     def t_SORT_DIRECTION(self, t):
         r',?\s*(/|\\)'
         t.value = t.value[-1]
@@ -49,6 +55,11 @@ class ExtendedJsonPathLexer(lexer.JsonPathLexer):
         t.type = self.reserved_words.get(t.value, 'ID')
         return t
 
+    def t_FLOAT(self, t):
+        r'-?\d+\.\d+'
+        t.value = float(t.value)
+        return t
+
 
 class ExtentedJsonPathParser(parser.JsonPathParser):
     """Custom LALR-parser for JsonPath"""
@@ -61,9 +72,12 @@ class ExtentedJsonPathParser(parser.JsonPathParser):
 
     def p_jsonpath_operator_jsonpath(self, p):
         """jsonpath : NUMBER operator NUMBER
+                    | FLOAT operator FLOAT
                     | ID operator ID
                     | NUMBER operator jsonpath
+                    | FLOAT operator jsonpath
                     | jsonpath operator NUMBER
+                    | jsonpath operator FLOAT
                     | jsonpath operator jsonpath
         """
 
@@ -101,7 +115,9 @@ class ExtentedJsonPathParser(parser.JsonPathParser):
     def p_expression(self, p):
         """expression : jsonpath
                       | jsonpath FILTER_OP ID
+                      | jsonpath FILTER_OP FLOAT
                       | jsonpath FILTER_OP NUMBER
+                      | jsonpath FILTER_OP BOOL
         """
         if len(p) == 2:
             left, op, right = p[1], None, None
@@ -161,3 +177,14 @@ class ExtentedJsonPathParser(parser.JsonPathParser):
 
 def parse(path, debug=False):
     return ExtentedJsonPathParser(debug=debug).parse(path)
+
+
+def match(pattern, data, **parse_kwargs):
+    """Returns all matched values of pattern in data"""
+    return [m.value for m in parse(pattern, **parse_kwargs).find(data)]
+
+
+def match1(pattern, data, **parse_kwargs):
+    """Returns first matched value of pattern in data or None if no matches"""
+    matches = match(pattern, data, **parse_kwargs)
+    return matches[0] if matches else None
