@@ -32,11 +32,20 @@ class TestJsonpath_rw_ext(testscenarios.WithScenarios,
     scenarios = [
         ('sorted_list', dict(string='objects.`sorted`',
                              data={'objects': ['alpha', 'gamma', 'beta']},
-                             target=['alpha', 'beta', 'gamma'])),
+                             target=[['alpha', 'beta', 'gamma']])),
+        ('sorted_list_indexed', dict(string='objects.`sorted`[1]',
+                                     data={'objects': [
+                                         'alpha', 'gamma', 'beta']},
+                                     target='beta')),
         ('sorted_dict', dict(string='objects.`sorted`',
                              data={'objects': {'cow': 'moo', 'horse': 'neigh',
                                                'cat': 'meow'}},
-                             target=['cat', 'cow', 'horse'])),
+                             target=[['cat', 'cow', 'horse']])),
+        ('sorted_dict_indexed', dict(string='objects.`sorted`[0]',
+                                     data={'objects': {'cow': 'moo',
+                                                       'horse': 'neigh',
+                                                       'cat': 'meow'}},
+                                     target='cat')),
 
         ('len_list', dict(string='objects.`len`',
                           data={'objects': ['alpha', 'gamma', 'beta']},
@@ -93,44 +102,85 @@ class TestJsonpath_rw_ext(testscenarios.WithScenarios,
                                               {'cow': 8, 'cat': 3}]},
                             target=[{'cow': 8, 'cat': 2},
                                     {'cow': 7, 'cat': 2}])),
-
+        ('filter_float_gt', dict(
+            string='objects[?confidence>=0.5].prediction',
+            data={
+                'objects': [
+                    {'confidence': 0.42,
+                     'prediction': 'Good'},
+                    {'confidence': 0.58,
+                     'prediction': 'Bad'},
+                ]
+            },
+            target=['Bad']
+        )),
         ('sort1', dict(string='objects[/cow]',
                        data={'objects': [{'cat': 1, 'cow': 2},
                                          {'cat': 2, 'cow': 1},
                                          {'cat': 3, 'cow': 3}]},
-                       target=[{'cat': 2, 'cow': 1},
+                       target=[[{'cat': 2, 'cow': 1},
                                {'cat': 1, 'cow': 2},
-                               {'cat': 3, 'cow': 3}])),
+                               {'cat': 3, 'cow': 3}]])),
+        ('sort1_indexed', dict(string='objects[/cow][0].cat',
+                               data={'objects': [{'cat': 1, 'cow': 2},
+                                                 {'cat': 2, 'cow': 1},
+                                                 {'cat': 3, 'cow': 3}]},
+                               target=2)),
         ('sort2', dict(string='objects[\cat]',
                        data={'objects': [{'cat': 2}, {'cat': 1}, {'cat': 3}]},
-                       target=[{'cat': 3}, {'cat': 2}, {'cat': 1}])),
+                       target=[[{'cat': 3}, {'cat': 2}, {'cat': 1}]])),
+        ('sort2_indexed', dict(string='objects[\cat][-1].cat',
+                               data={'objects': [{'cat': 2}, {'cat': 1},
+                                                 {'cat': 3}]},
+                               target=1)),
         ('sort3', dict(string='objects[/cow,\cat]',
                        data={'objects': [{'cat': 1, 'cow': 2},
                                          {'cat': 2, 'cow': 1},
                                          {'cat': 3, 'cow': 1},
                                          {'cat': 3, 'cow': 3}]},
-                       target=[{'cat': 3, 'cow': 1},
+                       target=[[{'cat': 3, 'cow': 1},
                                {'cat': 2, 'cow': 1},
                                {'cat': 1, 'cow': 2},
-                               {'cat': 3, 'cow': 3}])),
+                               {'cat': 3, 'cow': 3}]])),
+        ('sort3_indexed', dict(string='objects[/cow,\cat][0].cat',
+                               data={'objects': [{'cat': 1, 'cow': 2},
+                                                 {'cat': 2, 'cow': 1},
+                                                 {'cat': 3, 'cow': 1},
+                                                 {'cat': 3, 'cow': 3}]},
+                               target=3)),
         ('sort4', dict(string='objects[/cat.cow]',
                        data={'objects': [{'cat': {'dog': 1, 'cow': 2}},
                                          {'cat': {'dog': 2, 'cow': 1}},
                                          {'cat': {'dog': 3, 'cow': 3}}]},
-                       target=[{'cat': {'dog': 2, 'cow': 1}},
+                       target=[[{'cat': {'dog': 2, 'cow': 1}},
                                {'cat': {'dog': 1, 'cow': 2}},
-                               {'cat': {'dog': 3, 'cow': 3}}])),
+                               {'cat': {'dog': 3, 'cow': 3}}]])),
+        ('sort4_indexed', dict(string='objects[/cat.cow][0].cat.dog',
+                               data={'objects': [{'cat': {'dog': 1,
+                                                          'cow': 2}},
+                                                 {'cat': {'dog': 2,
+                                                          'cow': 1}},
+                                                 {'cat': {'dog': 3,
+                                                          'cow': 3}}]},
+                               target=2)),
         ('sort5_twofields', dict(string='objects[/cat.(cow,bow)]',
                                  data={'objects':
                                        [{'cat': {'dog': 1, 'bow': 3}},
                                         {'cat': {'dog': 2, 'cow': 1}},
                                         {'cat': {'dog': 2, 'bow': 2}},
                                         {'cat': {'dog': 3, 'cow': 2}}]},
-                                 target=[{'cat': {'dog': 2, 'cow': 1}},
+                                 target=[[{'cat': {'dog': 2, 'cow': 1}},
                                          {'cat': {'dog': 2, 'bow': 2}},
                                          {'cat': {'dog': 3, 'cow': 2}},
-                                         {'cat': {'dog': 1, 'bow': 3}}])),
+                                         {'cat': {'dog': 1, 'bow': 3}}]])),
 
+        ('sort5_indexed', dict(string='objects[/cat.(cow,bow)][0].cat.dog',
+                               data={'objects':
+                                     [{'cat': {'dog': 1, 'bow': 3}},
+                                      {'cat': {'dog': 2, 'cow': 1}},
+                                      {'cat': {'dog': 2, 'bow': 2}},
+                                      {'cat': {'dog': 3, 'cow': 2}}]},
+                               target=2)),
         ('arithmetic_number_only', dict(string='3 * 3', data={},
                                         target=[9])),
 
@@ -238,6 +288,46 @@ class TestJsonpath_rw_ext(testscenarios.WithScenarios,
             target=["cat-bow"]
         )),
 
+        ('bug-#2-correct', dict(
+            string='foo[?(@.baz==1)]',
+            data={'foo': [{'baz': 1}, {'baz': 2}]},
+            target=[{'baz': 1}],
+        )),
+
+        ('bug-#2-wrong', dict(
+            string='foo[*][?(@.baz==1)]',
+            data={'foo': [{'baz': 1}, {'baz': 2}]},
+            target=[],
+        )),
+
+        ('boolean-filter-true', dict(
+            string='foo[?flag = true].color',
+            data={'foo': [{"color": "blue", "flag": True},
+                          {"color": "green", "flag": False}]},
+            target=['blue']
+        )),
+
+        ('boolean-filter-false', dict(
+            string='foo[?flag = false].color',
+            data={'foo': [{"color": "blue", "flag": True},
+                          {"color": "green", "flag": False}]},
+            target=['green']
+        )),
+
+        ('boolean-filter-other-datatypes-involved', dict(
+            string='foo[?flag = true].color',
+            data={'foo': [{"color": "blue", "flag": True},
+                          {"color": "green", "flag": 2},
+                          {"color": "red", "flag": "hi"}]},
+            target=['blue']
+        )),
+
+        ('boolean-filter-string-true-string-literal', dict(
+            string='foo[?flag = "true"].color',
+            data={'foo': [{"color": "blue", "flag": True},
+                          {"color": "green", "flag": "true"}]},
+            target=['green']
+        )),
     ]
 
     def test_fields_value(self):
@@ -250,7 +340,20 @@ class TestJsonpath_rw_ext(testscenarios.WithScenarios,
         elif isinstance(self.target, (int, float)):
             self.assertEqual(self.target, result[0].value)
         else:
-            self.assertEqual(self.target, result.value)
+            self.assertEqual(self.target, result[0].value)
+
+    def test_shortcut_functions(self):
+        jsonpath.auto_id_field = None
+        parse_result = parser.parse(self.string, debug=True).find(self.data)
+        match_result = parser.match(self.string, self.data, debug=True)
+        match1_result = parser.match1(self.string, self.data, debug=True)
+
+        self.assertEqual(match_result, [r.value for r in parse_result])
+        if match_result:
+            self.assertEqual(match_result[0], match1_result)
+        else:
+            self.assertIsNone(match1_result)
+
 
 # NOTE(sileht): copy of tests/test_jsonpath.py
 # to ensure we didn't break jsonpath_rw
